﻿using System;
using System.Diagnostics;
using System.Management;
using DerpingDrivers.Exceptions;
using ExceptionReporting;
using Microsoft.Win32;

namespace DerpingDrivers.Util
{
    public static class BcdHelper
    {
        /// <summary>
        ///     Gets or sets the current value if BCDE_LIBRARY_TYPE_ALLOW_PRERELEASE_SIGNATURES from the default boot entry.
        /// </summary>
        public static bool AllowPrereleaseSignatures
        {
            get
            {
                var connectionOptions =
                    new ConnectionOptions
                    {
                        Impersonation = ImpersonationLevel.Impersonate,
                        EnablePrivileges = true
                    };

                var managementScope = new ManagementScope(@"root\WMI", connectionOptions);

                using (var bootMgrObj = new ManagementObject(managementScope,
                    new ManagementPath(
                        "root\\WMI:BcdObject.Id=\"{fa926493-6f1c-4193-a414-58f0b2456d1e}\",StoreFilePath=\"\""), null))
                {
                    var inParams = bootMgrObj.GetMethodParameters("GetElement");
                
                    inParams["Type"] = (uint) 0x16000049;
                    var outParams = bootMgrObj.InvokeMethod("GetElement", inParams, null);
                    var outObj = (ManagementBaseObject) outParams?.Properties["Element"].Value;

                    var allowPrereleaseSignatures = outObj != null && (bool) outObj.GetPropertyValue("Boolean");

                    return allowPrereleaseSignatures;
                }
            }

            set
            {
                try
                {
                    if (value)
                    {
                        // NOTE: haven't found an API for this (yet), so system() we go!
                        var cmd = new Process
                        {
                            StartInfo =
                            {
                                FileName = @"C:\WINDOWS\system32\bcdedit.exe",
                                Arguments = "-set TESTSIGNING ON",
                                WindowStyle = ProcessWindowStyle.Hidden,
                                CreateNoWindow = true,
                                UseShellExecute = false
                            }
                        };

                        cmd.Start();
                        cmd.WaitForExit();

                        if (cmd.ExitCode != 0)
                            throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't enable TESTSIGNING");
                    }
                    else
                    {
                        // NOTE: haven't found an API for this (yet), so system() we go!
                        var cmd = new Process
                        {
                            StartInfo =
                            {
                                FileName = @"C:\WINDOWS\system32\bcdedit.exe",
                                Arguments = "-set TESTSIGNING OFF",
                                WindowStyle = ProcessWindowStyle.Hidden,
                                CreateNoWindow = true,
                                UseShellExecute = false
                            }
                        };

                        cmd.Start();
                        cmd.WaitForExit();

                        if (cmd.ExitCode != 0)
                            throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't disable TESTSIGNING");
                    }
                }
                catch (Exception ex)
                {
                    var er = new ExceptionReporter
                    {
                        Config =
                        {
                            AppName = "DerpingDrivers",
                            SendMethod = ReportSendMethod.None
                        }
                    };
                    er.Show(ex);
                }
            }
        }
    }
}